@@ -1,5 +1,6 @@
 package Class::STAF;
 use strict;
+use threads::shared;
 use Class::STAF::Marshalled qw(:all);
 
 our $VERSION = 0.02;
@@ -15,16 +16,28 @@ our @EXPORT_OK = qw{
     get_staf_class_name
 };
 
+my %thread_ref_count;
+my $id_counter = 1;
+
 sub new {
     my ($class, $processName) = @_;
     require PLSTAF;
     my $handle = STAF::STAFHandle->new($processName);
     if ($handle->{rc} != $STAF::kOk) {
         $! = $handle->{rc};
-        $@ = ' ';
         return;
     }
-    return bless {handle=>$handle}, $class;
+    my $refcount : shared = 1;
+    my $key = $id_counter++;
+    $thread_ref_count{$key} = \$refcount;
+    return bless {handle => $handle, refcount => $key}, $class;
+}
+
+sub CLONE {
+    foreach (values %thread_ref_count) {
+        lock $$_;
+        $$_++;
+    }
 }
 
 sub submit {
@@ -32,7 +45,7 @@ sub submit {
     my $result = $self->{handle}->submit($location, $service, $request);
     if ($result->{rc} != $STAF::kOk) {
         $! = $result->{rc};
-        $@ = $result->{result};
+        $self->{LastError} = $result->{result};
         return;
     } 
     return $result->{result};
@@ -43,7 +56,7 @@ sub submit2 {
     my $result = $self->{handle}->submit2($syncOption, $location, $service, $request);
     if ($result->{rc} != $STAF::kOk) {
         $! = $result->{rc};
-        $@ = $result->{result};
+        $self->{LastError} = $result->{result};
         return;
     } 
     return $result->{result};
@@ -54,8 +67,21 @@ sub host {
     return Class::STAF::Host->new($self, $hostname);
 }
 
+sub LastError {
+    my $self = shift;
+    return $self->{LastError};
+}
+
 sub DESTROY {
     my $self = shift;
+    my $key = $self->{refcount};
+    {
+        my $ref = $thread_ref_count{$key};
+        lock $$ref;
+        $$ref--;
+        delete $thread_ref_count{$key};
+        return unless $$ref == 0;
+    }
     my $rc = $self->{handle}->unRegister();
     if ($rc != $STAF::kOk) {
         warn "Failed to unRegister from STAF";
@@ -81,6 +107,11 @@ sub submit2 {
     return $self->{Parent}->submit2($syncOption, $self->{Host}, $service, $request);
 }
 
+sub LastError {
+    my $self = shift;
+    return $self->{Parent}->{LastError};
+}
+
 sub service {
     my ($self, $service) = @_;
     return Class::STAF::Service->new($self->{Parent}, $self->{Host}, $service);
@@ -105,6 +136,11 @@ sub submit2 {
     return $self->{Parent}->submit2($syncOption, $self->{Host}, $self->{Service}, $request);
 }
 
+sub LastError {
+    my $self = shift;
+    return $self->{Parent}->{LastError};
+}
+
 1;
 
 __END__
@@ -117,21 +153,24 @@ Class::STAF - Simplify version for the Perl STAF API
 
     use Class::STAF;
     my $handle = Class::STAF->new("My Program")
-        or die "Error $!: $@";
+        or die "Error: can not create handle. RC=$!";
     
     my $result = $handle->submit("local", "PING", "PING")
-        or print "submit failed ($!): $@\n";
+        or print "submit failed ($!): ", $handle->LastError, "\n";
     
     $service = $handle->host("local")->service("PING");
     $result = $service->submit("PING");
 
 =head1 DESCRIPTION
 
-This module is an alternative API for STAF. because frankly, the current one is ugly.
+This module is an alternative API for STAF - More Perlish.
+
+For more info about STAF: http://staf.sourceforge.net/
+
 Instead of checking for every request that the return code is zero, and only then
 proceed, this API return the answer immidiatly. Only if the return code is not zero,
 the submit will return undef. Then the return code is saved in $!, and the error message
-is in $@.
+can be retrived using $handle->LastError command.
 
 Also export by default the Marshall and UnMarshall functions from L<Class::STAF::Marshalled>,
 and will export by request the get_staf_fields and get_staf_class_name.
@@ -142,7 +181,9 @@ The functions are similar to the original STAF API.
 Creating:
 
     my $handle = Class::STAF->new("My Program")
-        or die "Error $!: $@";
+        or die "Error: can not create handle. RC=$!";
+
+The new function only return a return code.
 
 Member functions:
     
@@ -169,11 +210,21 @@ And use it:
 
     $service->submit("PING") or die "Ping is not working on that host?!";
 
+=head1 Thread Safety
+
+This module is thread safe by itself, but it is based on PLSTAF. It is still not clear
+wether the PLSTAF module is thread safe or not.
+
+Also, this warpper will automatically unregister the STAF Handle only after
+it is released from all the threads that use it.
+
+As result of this thread safety, this module support Perl 5.6.1 and up.
+
 =head1 BUGS
 
 Non known.
 
-This is a first release - your feedback will be appriciated.
+This is a first release - your feedback will be appreciated.
 
 =head1 SEE ALSO
 
